#!/usr/bin/perl -w

#call: rotemMethodV2.pl annotation bedfile outfile

#format annotation: chr db type start end . strand 1 comment
#bedfile format: CHR annotated_end an_end+1 afterlen comment strand cov

#bedfile is the output of getFragLenReps.pl

#for each annotation, get all positions in bedfile that are
#-on the same strand
#-and (pos-insertsize) < annotated 3'end
#startinsertlen---gene....3'---pos-endinsertlen

#output format is the same as bedfile, but now less entries
#bedfile format: CHR annotated_end an_end+1 afterlen cov strand

use Data::Dumper;
use strict;
use warnings;


my $anno = shift;
my $bedfile = shift; 
my $outfile = shift;

open(my $outf, ">>", $outfile);

my %endsp = (); #end coord to insert size list, sep by ';'
my %endsm = ();

my $chr;

open FA,"<$anno" or die "can't open $anno\n";
while(<FA>){
    chomp;
    my $l1 = $_;
    if($l1 =~ /^#/){next;}
    my @F = split '\t', $l1;
    $chr = $F[0];
        my $s;
        my $e;
        if($F[3] < $F[4]){
            $s = $F[3];
            $e = $F[4];
        }
        else{
            $s = $F[4];
            $e = $F[3];
        }
	my $strand = $F[6];
	if($strand eq "+"){
	    $endsp{$e}="";
	}
	else{
	    $endsm{$e}="";
	}
}

open FS1,"<$bedfile" or die "can't open $bedfile\n";
while(<FS1>){
    chomp;
    my $line = $_;
    if($line =~ /^@/){next;}
    if($line =~ /^#/){next;}
    my @G = split '\t', $line;
    #bedfile format: CHR annotated_end an_end+1 afterlen comment strand cov
    $chr = $G[0];
    my $end = $G[1];
    my $ilen = $G[3];
    my $strand = $G[5];
    my $cov = $G[6];

    #check annotation overlap
    #fwd strand
    if($strand eq "+"){
	for my $k (keys %endsp){
	    if($k <= $end && $k + $ilen >= $end){
		my $str = "$end\;$ilen\;$cov";
		if($endsp{$k} eq ""){
		    $endsp{$k} = $str;
		}
		else{
		    my @H = split ';', $endsp{$k};
		    if($cov >= $H[2]){$endsp{$k} = $str;}
		}

	    }
	    else{next;}
	}
    }
    else{
	for my $k (keys %endsm){
	    if($k >= $end && $end + $ilen >= $k){
		my $str = "$end\;$ilen\;$cov";
		if($endsm{$k} eq ""){
		    $endsm{$k} = $str;
		}
		else{
		    my @H = split ';', $endsm{$k};
		    if($cov >= $H[2]){$endsm{$k} = $str;}
		}

	    }
	    else{next;}
	}

    }
}


#outfile format: CHR annotated_end an_end+1 afterlen comment strand
my $pnum = scalar (keys %endsp);
foreach my $kp (sort (keys %endsp)){
    if($endsp{$kp} eq ""){next;}
    my @ins = split ';', $endsp{$kp};
    my $kp1 = $ins[0]+1;
    print $outf "$chr\t$ins[0]\t$kp1\t$ins[1]\t$ins[2]\t+\n";
}

my $mnum = scalar (keys %endsm);
foreach my $km (keys %endsm){
    if($endsm{$km} eq ""){next;}
    my @ins = split ';', $endsm{$km};
    my $km1 = $ins[0]+1;
    print $outf "$chr\t$ins[0]\t$km1\t$ins[1]\t$ins[2]\t-\n";
}
