#!/usr/bin/perl

use strict;
use warnings;
use Data::Dumper;
use Getopt::Long;

my $pvalue_th   = 0.05;    
my $min_seen_th = 'all';

GetOptions(
  "pv=f"         => \$pvalue_th,
  "ms=i"         => \$min_seen_th
);

my @input_fh = @ARGV;

if ($min_seen_th eq "all"){
  $min_seen_th = scalar(@input_fh);
}

my %data = ();

foreach my $input_fh (@input_fh){
  open FH, "< $input_fh" or die "can t open $input_fh\n";
  while(<FH>){
    chomp;
    my @F = split"\t",$_;
    
    next if ($F[0] eq "readType" || $F[2] eq "chromStart");
    
    my ($readType, $chrom, $chromStart, $chromEnd, $strand, $readCounts, $inSources, $Ratio, $pValue) = @F;
    my $positionalID = join("::", $chrom, $chromEnd, $strand);
    
    my %sigData = ();
    
    $sigData{chrom}      = $chrom ;
    $sigData{position}   = $chromEnd ;
    $sigData{strand}     = $strand ;
    $sigData{readCounts} = $readCounts ;
    $sigData{inSources}  = $inSources ;  
    $sigData{pValue}     = $pValue ;
    $sigData{Ratio}      = $Ratio ;
    
    push @{$data{$positionalID}}, \%sigData;
    
  }
  close FH;
}
  
print join("\t", '#chromosome', '#start', '#stop', '#strand', '#numer_of_significant_samples', '#mean_number_of_suportive_reads', '#geometricmean_of_pValues')."\n";
foreach my $positionalID (sort keys %data){
  my $seen_sigs = scalar(@{$data{$positionalID}});
  if($seen_sigs >= $min_seen_th){
      
    my @ratios   = ();
    my @counts   = ();
    my @pvalues  = ();
    my $chrom    = '';
    my $strand   = '';
    my $position = '';    
    
    foreach my $index (0 .. $#{$data{$positionalID}}){
      push @ratios, $data{$positionalID}->[$index]->{Ratio};
      push @counts, $data{$positionalID}->[$index]->{readCounts};
      push @pvalues, $data{$positionalID}->[$index]->{pValue};
      $chrom    = $data{$positionalID}->[$index]->{chrom};
      $strand   = $data{$positionalID}->[$index]->{strand};
      $position = $data{$positionalID}->[$index]->{position};
    }
    
    my $gmean_pvalue = &gmean(@pvalues);
    my $gmean_ratios = &gmean(@ratios);
    my $mean_coverage = &mean(@counts);
    
    if ($gmean_pvalue <= $pvalue_th ){
      print join("\t", $chrom, $position-1, $position, $strand, scalar(@{$data{$positionalID}}), $mean_coverage, $gmean_pvalue)."\n";
    }
  }
}


sub mean {
  my @val = @_;

  my $sum = 0;
  my $n   = 0;

  foreach my $val (@val){
    $sum += $val;
    $n++;
  }

  return($sum/$n);
}

sub gmean {
  my @val = @_;

  if(scalar(@val)<1){
    return('NaN');
  }
  
  my $product = 1;
  my $n       = 0;

  foreach my $val (@val){
    if($val > 1/1000){
      $product *= $val;
    }
    else{
      $product *= 1/1000;
    }
   $n++;
  }
  return($product**(1/$n));
}


